use cgmath::{Deg, InnerSpace, Point2, Vector2};
use galactica_content as content;
use galactica_render::{AnchoredUiPosition, UiSprite};
use galactica_world::{util, ShipPhysicsHandle, World};

// TODO: camera as one unit
pub fn build_radar(
	ct: &content::Content,
	player: &ShipPhysicsHandle,
	physics: &World,
	camera_zoom: f32,
	camera_aspect: f32,
) -> Vec<UiSprite> {
	let mut out = Vec::new();

	let radar_range = 2000.0;
	let radar_size = 300.0;

	out.push(UiSprite {
		texture: ct.get_texture_handle("ui::radar"),
		pos: AnchoredUiPosition::NwNw(Point2 { x: 10.0, y: -10.0 }),
		dimensions: Point2 {
			x: radar_size,
			y: radar_size,
		},
		angle: Deg(0.0),
	});

	// Draw viewport frame
	let d = Vector2 {
		x: (camera_zoom / 2.0) * camera_aspect,
		y: camera_zoom / 2.0,
	};
	let m = d.magnitude() / radar_range;
	let d = d / radar_range * (radar_size / 2.0);
	if m < 0.8 {
		let texture = ct.get_texture_handle("ui::radarframe");
		let dimensions = Point2 {
			x: texture.aspect,
			y: 1.0,
		} * 7.0f32.min((0.8 - m) * 70.0);
		out.push(UiSprite {
			texture,
			pos: AnchoredUiPosition::NwNw(Point2 {
				x: (radar_size / 2.0 + 10.0) - d.x,
				y: (radar_size / -2.0 - 10.0) + d.y,
			}),
			dimensions,
			angle: Deg(0.0),
		});

		out.push(UiSprite {
			texture,
			pos: AnchoredUiPosition::NwSw(Point2 {
				x: (radar_size / 2.0 + 10.0) - d.x,
				y: (radar_size / -2.0 - 10.0) - d.y,
			}),
			dimensions,
			angle: Deg(90.0),
		});

		out.push(UiSprite {
			texture,
			pos: AnchoredUiPosition::NwSe(Point2 {
				x: (radar_size / 2.0 + 10.0) + d.x,
				y: (radar_size / -2.0 - 10.0) - d.y,
			}),
			dimensions,
			angle: Deg(180.0),
		});

		out.push(UiSprite {
			texture,
			pos: AnchoredUiPosition::NwNe(Point2 {
				x: (radar_size / 2.0 + 10.0) + d.x,
				y: (radar_size / -2.0 - 10.0) + d.y,
			}),
			dimensions,
			angle: Deg(270.0),
		});
	}

	let (_, pr) = physics.get_ship_body(player).unwrap();
	let pr = util::rigidbody_position(pr);
	let texture = ct.get_texture_handle("ui::blip");
	for (s, r) in physics.iter_ship_body() {
		let p = util::rigidbody_position(r);
		let d = p - pr;
		let m = d.magnitude() / radar_range;
		let angle: Deg<f32> = util::rigidbody_rotation(r)
			.angle(Vector2 { x: 0.0, y: 1.0 })
			.into();
		if s.physics_handle == *player {
			out.push(UiSprite {
				texture,
				pos: AnchoredUiPosition::NwC(Point2 {
					x: radar_size / 2.0 + 10.0,
					y: radar_size / -2.0 - 10.0,
				}),
				dimensions: Point2 {
					x: texture.aspect,
					y: 1.0,
				} * 5.0f32.min((0.8 - m) * 50.0),
				angle: -angle,
			});
		} else if m < 0.8 {
			out.push(UiSprite {
				texture,
				pos: AnchoredUiPosition::NwC(
					Point2 {
						x: radar_size / 2.0 + 10.0,
						y: radar_size / -2.0 - 10.0,
					} + (d / radar_range * (radar_size / 2.0)),
				),
				dimensions: Point2 {
					x: texture.aspect,
					y: 1.0,
				} * 5.0f32.min((0.8 - m) * 50.0),
				angle: -angle,
			});
		}
	}

	return out;
}
